import os
import yaml

import simulation
from flockai.patterns.singleton import Singleton


class SimulationConfig(metaclass=Singleton):

    def __init__(self):
        self.config_file = os.path.abspath(simulation.__file__)
        self.data = None
        self.simulation_enabled = False
        self._read_config()
        self._update_config()

    def _read_config(self):
        stream = open(self.config_file, 'r')
        self.data = yaml.safe_load(stream)

        self.simulation_id = self.data['simulation_id']
        self.simulation_image_directory = f"{self.data['image_directory']}{str(self.simulation_id)}/"

    def _update_config(self):
        self.data['simulation_id'] = self.simulation_id + 1
        with open(self.config_file, 'w') as f:
            f.write(yaml.dump(self.data, default_flow_style=False))

    def create_simulation_image_directory(self) -> str:
        try:
            os.makedirs(self.simulation_image_directory)
        except OSError:
            print("Creation of directory failed")

    def get_simulation_id(self):
        return self.simulation_id

    def get_simulation_image_directory(self) -> str:
        return self.simulation_image_directory



class Singleton(type):
    __instance = None

    def __call__(cls, *args, **kwargs):
        if cls.__instance is None:
            cls.__instance = super(Singleton, cls).__call__(*args, **kwargs)
        return cls.__instance



import os
import json
from functools import wraps
from time import time, sleep
from random import randint

from flockai.PyCatascopia.Metrics import Metric, SimpleMetric


class CatascopiaDecorators:

    @classmethod
    def timeit(cls, f):
        @wraps(f)
        def wrap(*args, **kw):
            ts = time()
            result = f(*args, **kw)
            te = time()
            name, unit, desc = "time__%s" % (f.__name__), 's', 'Execution duration from %s' % (f.__name__)
            simple_metric = SimpleMetric(name, unit, desc, minVal=0.0, higherIsBetter=False)
            simple_metric.set_val(te - ts)
            cls.__store(simple_metric)
            return result
        return wrap

    @classmethod
    def __store(cls, metric: Metric,
                fpath='.'+os.sep + 'time_decorated_metrics.jsonl', fmode='a', encoding='utf-8', format='json'):
        with open(fpath, mode=fmode, encoding=encoding) as file:
            if format == 'json':
                s = json.dumps(metric.to_dict()) + os.linesep
            else:
                s = str(metric) + os.linesep
            file.write(s)


@CatascopiaDecorators.timeit
def intensive_workload_function():
    sleep(randint(1, 10))


def main():

    intensive_workload_function()


if __name__ == "__main__":
    main()

